﻿Type=Class
Version=4.3
ModulesStructureVersion=1
B4i=true
@EndOfDesignText@
'ValiDialog class - implements a number of different kinds of dialog (message and input) with optional validation on the input dialogs

#Event: Dialog_Result (Result as Int)

Sub Class_Globals
	Private const ClassName As String = "ValiDialog"
	
	Private myCallback As Object
	Dim myEventPrefix As String

	Private const DIALOGTYPES() As String = Array As String( _
		"Toast", "Message")
	Private myDialogType As Int
	
	Private myDlg As CustomLayoutDialog
	Private myPanel As Panel
	Private vxScrollView As B4XView
	Private vxMessage As B4XView
	
	Private wDialog As Float
	Private hDialog As Float
	Private wScrollView As Float
	Private hScrollView As Float
	Private wContent As Float
	Private hContent As Float
	Private hMessage As Float
	Private hInput As Float
#If B4a
	'Dialog measurements
	Private const hDlgAbove As Float = 32dip 'ignore Height needed to align top of icon to top of display
	Private const hDlgAround As Float = 16dip 'Reduce dialog width/height to improve appearance
	Private const wDlgAround As Float = 16dip 'Reduce dialog width/height to improve appearance
	Private const hDlgIcon As Float = 40dip 'Height of the Icon at the top of the dialog
	Private const hDlgTitle As Float = 40dip 'Height of the Title with Icon = NULL
	Private const hDlgTitleIcon As Float = 40dip 'Height of the Title plus Icon
	Private const hDlgButtons As Float = 80dip 'Height of the button area
	Private const hDlgTopBottom As Float = 8dip 'Top/Bottom margins around the content area
	Private const wDlgLeftRight As Float = 8dip 'Left/Right margins around the content area
	'Content measurements
	Private const hCntTopBottom As Float = 8dip 'Top/Bottom margins inside the content area
	Private const hCntMiddle As Float = 8dip 'Middle margin (between message and input control) if needed
	Private const wCntLeftRight As Float = 8dip 'Left/Right margins inside the content area
#Else If B4i
	'Dialog measurements
	Private const hDlgAbove As Float = 32dip 'Height needed to align top of icon to top of display
	Private const hDlgAround As Float = 0dip 'Reduce dialog width/height to improve appearance
	Private const wDlgAround As Float = 16dip 'Reduce dialog width/height to improve appearance
	Private const hDlgIcon As Float = 44dip 'Height of the Icon at the top of the dialog
	Private const hDlgTitle As Float = 116dip 'Height of the Title with Icon = NULL
	Private const hDlgTitleIcon As Float = 116dip 'Height of the Title plus Icon
	Private const hDlgButtons As Float = 60dip 'Height of the button area
	Private const hDlgTopBottom As Float = 0dip 'Top/Bottom margins around the content area
	Private const wDlgLeftRight As Float = 0dip 'Left/Right margins around the content area
	'Content measurements
	Private const hCntTopBottom As Float = 8dip 'Top/Bottom margins inside the content area
	Private const hCntMiddle As Float = 8dip 'Middle margin (between message and input control) if needed
	Private const wCntLeftRight As Float = 8dip 'Left/Right margins inside the content area
#End If
	
 	Private myMessage As CSBuilder
	Private myTitle As String
	Private myPositiveText As CSBuilder
	Private myNegativeText As CSBuilder
	Private myCancelText As CSBuilder
	Private myTag As Object
	
End Sub

#Region Setup/ShowToast/Close
'Set callback info, DialogType and initial Message value (CSBuilder or String)
Public Sub Initialize(Callback As Object, EventPrefix As String, DialogType As Int, Message As Object) As ValiDialog
	myCallback = Callback
	myEventPrefix = EventPrefix
	If DialogType < 0 Or DialogType > ValiDialogConst.DIALOGTYPE_MAX Then RaiseException($"${ClassName}.Initialize"$, $"Invalid DialogType: ${DialogType}"$)
	myDialogType = DialogType
	setMessage(Message)
	myPositiveText.Initialize
	myNegativeText.Initialize
	myCancelText.Initialize
	Return Me
End Sub

'Shows a message using ToastMessageShow (Android) or iHUD (IOS)
public Sub ShowToast(LongDuration As Boolean)
	'Check ValiDialog type
	Select myDialogType
	Case ValiDialogConst.DIALOGTYPE_TOAST
	Case Else
		RaiseException($"${ClassName}.ShowToast"$, $"Should only be called for 'Toast' DialogType (Dialog Type used was ${DIALOGTYPES(myDialogType)})"$)
	End Select
	'Show the message
#If B4A
	ToastMessageShow(myMessage, LongDuration)
#Else If B4i
	Dim Toast As HUD
	Toast.ToastMessageShow(myMessage.ToString, LongDuration)
#End If
End Sub

'Builds and shows an aSync dialog
Public Sub ShowAsync(Cancelable As Boolean)
	'Check ValiDialog type
	Select myDialogType
	Case ValiDialogConst.DIALOGTYPE_TOAST
		RaiseException($"${ClassName}.ShowAsync"$, $"Should not be called for 'Toast' DialogType"$)
	Case Else
	End Select
	'Calculate widths/heights for panel/scrollview - top down. This sets maximum size for scrollview content.
	hDialog = Min(GetDeviceLayoutValues.Width, GetDeviceLayoutValues.Height) - (2 * hDlgAround)
	wDialog = Min(GetDeviceLayoutValues.Width, GetDeviceLayoutValues.Height) - (2 * wDlgAround)
	wScrollView = wDialog - (2 * wDlgLeftRight)
	hScrollView = hDialog - (2 * hDlgTopBottom)
	If myTitle.Length > 0 Then hScrollView = hScrollView - hDlgTitle
	If myPositiveText.Length > 0 Or myNegativeText.Length > 0 Or myCancelText.Length > 0 Then hScrollView = hScrollView - hDlgButtons
	'Calculate widths/heights for scrollview content - bottom up.
	wContent = wScrollView - (2 * wCntLeftRight)
	hInput = 0
	If myMessage.ToString.Length > 0 Then
		Dim lblMessage As Label
		lblMessage.Initialize("")
		vxMessage = lblMessage
#If B4i
		lblMessage.Multiline = True
		lblMessage.AttributedText = myMessage
#Else If B4A
		lblMessage.SingleLine = False
		lblMessage.Text = myMessage
#End If
		Dim MessageMetrics As SizeToFit_Metrics = SizeToFit.MeasureExpectedSize(lblMessage, wContent, 0)
		hMessage = MessageMetrics.Height
	End If
	hContent = hCntTopBottom + hMessage + hInput + hCntTopBottom
	If hMessage > 0 And hInput > 0 Then hContent = hContent + hCntMiddle
	hScrollView = Min(hScrollView, hContent)
	'Work content size back to dialog size
	hDialog = hScrollView + (2 * hDlgTopBottom)
	wDialog = wScrollView + (2 * wDlgLeftRight)
#If B4a
	'Android panel sizing includes icon, title, buttons and side margins
	If myTitle.Length > 0 Then hDialog = hDialog + hDlgTitle
	If myPositiveText.Length > 0 Or myNegativeText.Length > 0 Or myCancelText.Length > 0 Then hDialog = hDialog + hDlgButtons
	wDialog = wDialog + 32dip
#End If
	'Setup views
	myPanel.Initialize("")
	Dim sv As ScrollView
#If B4i
	sv.Initialize("vxScrollView", wScrollView, hContent)
#Else If B4A
	sv.Initialize2(hContent, "vxScrollView")
#End If
	vxScrollView = sv
	myPanel.AddView(vxScrollView, 0, 0, wScrollView, hScrollView)
	Dim hTop As Float = 0
	If hMessage > 0 Then
		sv.Panel.AddView(vxMessage, wCntLeftRight, hTop + hCntTopBottom, wContent, hMessage)
		hTop = hTop + hCntTopBottom + hMessage
	End If
#If B4i
	myPanel.SetLayoutAnimated(200, 1, 0, 0, wScrollView, hScrollView)
	myDlg.Initialize(myPanel)
	myDlg.ShowAsync(myTitle, myPositiveText.ToString, myCancelText.ToString, myNegativeText.ToString, Cancelable)
	MoveCustomDialog(myDlg, hDlgAbove) 'Move the dialog to top of display with icon still fully showing
#Else if B4a
	myPanel.SetLayoutAnimated(200, 0, 0, wScrollView, hScrollView)
	'Show dialog/adjust buttons & size
	myDlg.ShowAsync(myTitle, myPositiveText.ToString, myCancelText.ToString, myNegativeText.ToString, Null, Cancelable)
	myDlg.SetSize(wDialog, hDialog)
	Wait For Dialog_Ready (dp As Panel)
'	Log($"Dialog_Ready (${dp.Width},${dp.Height})"$)
	dp.AddView(myPanel, wDlgLeftRight, hDlgTopBottom, wScrollView, hScrollView)
#End If
	If myPositiveText.ToString.Length > 0 Then SetButtonText(myDlg.GetButton(ValiDialogConst.RESULT_POSITIVE), myPositiveText, Null, Null)
	If myNegativeText.ToString.Length > 0 Then SetButtonText(myDlg.GetButton(ValiDialogConst.RESULT_NEGATIVE), myNegativeText, Null, Null)
	If myCancelText.ToString.Length > 0 Then SetButtonText(myDlg.GetButton(ValiDialogConst.RESULT_CANCEL), myCancelText, Null, Null)
	'Wait for user input
	Wait For Dialog_Result (Result As Int)
'	Log($"Dialog_Result"$)
	CallSub2(myCallback, myEventPrefix & "_Result", Result)
#If B4i
	myPanel.RemoveViewFromParent
#Else If B4a
	myPanel.RemoveView
#End If
End Sub

'Close the dialog, raising an event with the specified Response type
public Sub CloseDialog(Result As Int)
	Log($"CloseDialog: ${Result}"$)
	'Check ValiDialog type
	Select Result
	Case ValiDialogConst.RESULT_POSITIVE, ValiDialogConst.RESULT_NEGATIVE, ValiDialogConst.RESULT_CANCEL
		myDlg.CloseDialog(Result)
	Case Else
		RaiseException($"${ClassName}.CloseDialog"$, $"Invalid Response: ${Result}"$)
	End Select
	CallSub2(myCallback, myEventPrefix & "_Result", Result)
End Sub
#End Region

#Region Properties
'Property: DialogType - get Dialog Type
public Sub getDialogType As Int
	Return myDialogType
End Sub

'Property: Message - gets/set Message as a String or CS Builder
public Sub getMessage As CSBuilder
	Return myMessage 'Always return a CSBuilder - other subs will use as-is or convert to String if needed
End Sub
public Sub setMessage(NewValue As Object)
	andMessage(NewValue)
End Sub
public Sub andMessage(NewValue As Object) As ValiDialog
	If NewValue Is String Then
		myMessage.Initialize.Append(NewValue).PopAll
	Else If NewValue Is CSBuilder Then
		myMessage = NewValue
	Else
		RaiseException($"${ClassName}.Message"$, $"Expecting String or CSBuilder, received ${GetType(NewValue)}."$)
	End If
	Return Me
End Sub

'Property: Title - gets/set Title as a String
public Sub getTitle As String
	Return myTitle
End Sub
public Sub setTitle(NewValue As String)
	andTitle(NewValue)
End Sub
public Sub andTitle(NewValue As String) As ValiDialog
	myTitle = NewValue
	Return Me
End Sub

'Property: PositiveText - gets/set PositiveText as a String or CS Builder
public Sub getPositiveText As CSBuilder
	Return myPositiveText 'Always return a CSBuilder - other subs will use as-is or convert to String if needed
End Sub
public Sub setPositiveText(NewValue As Object)
	andPositiveText(NewValue)
End Sub
public Sub andPositiveText(NewValue As Object) As ValiDialog
	If NewValue Is String Then
		myPositiveText.Initialize.Append(NewValue).PopAll
	Else If NewValue Is CSBuilder Then
		myPositiveText = NewValue
	End If
	Return Me
End Sub

'Property: NegativeText - gets/set NegativeText as a String or CS Builder
public Sub getNegativeText As CSBuilder
	Return myNegativeText 'Always return a CSBuilder - other subs will use as-is or convert to String if needed
End Sub
public Sub setNegativeText(NewValue As Object)
	andNegativeText(NewValue)
End Sub
public Sub andNegativeText(NewValue As Object) As ValiDialog
	If NewValue Is String Then
		myNegativeText.Initialize.Append(NewValue).PopAll
	Else If NewValue Is CSBuilder Then
		myNegativeText = NewValue
	End If
	Return Me
End Sub

'Property: CancelText - gets/set CancelText as a String or CS Builder
public Sub getCancelText As CSBuilder
	Return myCancelText 'Always return a CSBuilder - other subs will use as-is or convert to String if needed
End Sub
public Sub setCancelText(NewValue As Object)
	andCancelText(NewValue)
End Sub
public Sub andCancelText(NewValue As Object) As ValiDialog
	If NewValue Is String Then
		myCancelText.Initialize.Append(NewValue).PopAll
	Else If NewValue Is CSBuilder Then
		myCancelText = NewValue
	End If
	Return Me
End Sub

'Property: Tag - gets/set Tag
public Sub getTag As Object
	Return myTag
End Sub
public Sub setTag(NewValue As Object)
	andTag(NewValue)
End Sub
public Sub andTag(NewValue As Object) As ValiDialog
	myTag = NewValue
	Return Me
End Sub
#End Region

#Region Utility
'Set button text using non-null AttributedText objects 
private Sub SetButtonText(b As Button, NormalText As Object, HighlightedText As Object, DisabledText As Object)
#If B4i
   Dim no As NativeObject = b
   If Not(NormalText = Null) Then no.RunMethod("setAttributedTitle:forState:", Array(NormalText, 0))
   If Not(HighlightedText = Null) Then no.RunMethod("setAttributedTitle:forState:", Array(HighlightedText, 1))
   If Not(DisabledText = Null) Then no.RunMethod("setAttributedTitle:forState:", Array(DisabledText, 2))
#Else if B4A
   If Not(NormalText = Null) Then b.Text = NormalText
#End if
End Sub

#If B4i
private Sub MoveCustomDialog(cd As CustomLayoutDialog, Top As Int)
	Dim no As NativeObject = cd
	Dim v As View = no.GetField("alertView").GetField("view")
	Do While v.Top = 0
		Sleep(5)
	Loop
	v.Top = Top
End Sub
#End If

'Logs exception information then raises an exception to abort execution
Private Sub RaiseException(Location As String, Description As String)
	Log($"*** ERROR *** Location='${Location}' Description='${Description}'"$)
	If LastException.IsInitialized Then
#If B4i
		Log($"Last Exception: Error ${LastException.Code} '${LastException.Message}' in ${LastException.Domain}"$)
#Else
		Log($"Last Exception: '${LastException.Message}'"$)
#End If
	End If
	Dim i As Int = "A" 'ignore - Causes an exception whatever the platform
End Sub
#End Region